@@ -7,11 +7,12 @@ from datetime import datetime
 USE_TORCH = 0
 
 import bokeh.plotting as bp
-from bokeh.models import HoverTool, ColumnDataSource, Span
+from bokeh.models import HoverTool, ColumnDataSource, Span, CustomJSHover
 from bokeh.io import output_notebook, push_notebook
 from bokeh.layouts import layout
 from bokeh.resources import INLINE
 import numpy as np
+import pandas as pd
 
 if USE_TORCH:
     import torch
@@ -25,7 +26,7 @@ import matplotlib.cm as cm
 
 #from .parser import parse
 
-__version__ = '0.1.11'
+__version__ = '0.1.13'
 
 output_notebook(resources=INLINE)
 #output_notebook()
@@ -129,7 +130,6 @@ def parse_spec(spec):
 
 def parse3(x, y, spec): # -> list of (x, y, spec)
     tr = []
-    #import ipdb; ipdb.set_trace()
     if is_2d(y):
         h, w = len(y), len(y[0])
         if isinstance(x, Missing) and w == 2 and h > 2:
@@ -199,6 +199,12 @@ def parse(*args, color=None, legend=None):
             x, y = args
         if isinstance(y, dict):
             x, y = list(y.keys()), list(y.values())
+        elif isinstance(y, pd.DataFrame):
+            if legend is None:
+                legend = y.columns.values
+            x, y = y.index.values, y.T.values
+        elif isinstance(y, pd.Series):
+            x, y = y.index.values, y.values
         tr.extend(parse3(x, y, style))
     elif len(args) % 3 == 0:
         n = len(args)//3
@@ -220,16 +226,21 @@ def parse(*args, color=None, legend=None):
     # legend
     if isinstance(legend, (list, tuple)):
         if len(legend) != n:
-            raise ValueError(f'len(legend)={len(legend)}; legend should either be a string or a tuple of length {n}')
+            raise ValueError(f'len(legend)={len(legend)}; legend should either be a string or a list/tuple/ndarray of length {n}')
+        legends = legend
+    elif isinstance(legend, np.ndarray):
+        if legend.shape != (n,):
+            raise ValueError(f'legend.shape={legend.shape}; legend shape should be ({n},)')
+        if not isinstance(legend[0], str):
+            raise ValueError(f'legend={legend}; legend elements should be strings')
         legends = legend
     elif isinstance(legend, str):
         legends = [legend] * n
     elif legend is None:
         legends = [None] * n
     else:
-        raise ValueError(f'legend={legend}; it should either be a string or a tuple of length {n}')
+        raise ValueError(f'legend={legend}; it should either be a string or a list/tuple/ndarray of length {n}')
     qu = []
-    is_x_datetime = None
     for (x, y, spec), color, legend in zip(tr, colors, legends):
         style, _color = parse_spec(spec)
         if _color != 'a':
@@ -295,7 +306,7 @@ def plot(*args, p=None, hover=False, mode='plot', hline=None, vline=None, color=
                     if 'x_axis_type' in kw and kw['x_axis_type'] is not None:
                         raise ValueError('datetime x values is incompatible with "%s"' % mode)
                     else:
-                        x_axis_type = 'datetime'
+                        kw['x_axis_type'] = 'datetime'
                 p = figure(**kw)
                 FIGURE.append(p)
 #                print('A', FIGURE)
@@ -309,7 +320,11 @@ def plot(*args, p=None, hover=False, mode='plot', hline=None, vline=None, color=
 
         notebook_handle = kwargs.pop('notebook_handle', False)
         if hover:
-            p.add_tools(HoverTool(tooltips = [("x", "@x"),("y", "@y")]))
+            if is_dt:
+                p.add_tools(HoverTool(tooltips=[('x', '@x{%F}'), ('y', '@y'), ('name', '$name')],
+                            formatters={'@x': 'datetime'}))#, '@y': lat_custom}))
+            else:
+                p.add_tools(HoverTool(tooltips = [("x", "@x"),("y", "@y")]))
         for x, y, style, color_str, legend_i in quintuples:
             color = get_color(color_str)
             if isinstance(y, torch.Tensor):
@@ -323,15 +338,20 @@ def plot(*args, p=None, hover=False, mode='plot', hline=None, vline=None, color=
             source = ColumnDataSource(data=dict(x=x, y=y))
             legend_set = False
             if not style or '-' in style:
-                kw = kwargs
-                if legend_i is not None:
-                    kwargs['legend_label'] = legend_i
+                kw = kwargs.copy()
+                if legend_loc != 'hide' and legend_i is not None:
+                    kw['legend_label'] = legend_i
+                if hover:
+                    kw['name'] = legend_i
                 p.line('x', 'y', source=source, color=color, **kw)
                 legend_set = True
             if '.' in style:
+                kw = kwargs.copy()
                 legend_j = None if legend_set else legend_i
-                if legend_j is not None:
-                    kwargs['legend_label'] = legend_j
+                if legend_loc != 'hide' and legend_j is not None:
+                    kw['legend_label'] = legend_j
+                if hover:
+                    kw['name'] = legend_i
                 p.circle('x', 'y', source=source, color=color, **kw)
         if isinstance(hline, (int, float)):
             span = Span(location=hline, dimension='width', line_color=color, line_width=1, level='overlay')
@@ -339,10 +359,11 @@ def plot(*args, p=None, hover=False, mode='plot', hline=None, vline=None, color=
         elif isinstance(vline, (int, float)):
             span = Span(location=vline, dimension='height', line_color=color, line_width=1, level='overlay')
             p.renderers.append(span)
-        if legend is not None:
-            p.legend.click_policy="hide"
-        if legend_loc is not None:
-            p.legend.location = legend_loc
+        if legend_loc != 'hide':
+            if legend is not None:
+                p.legend.click_policy="hide"
+            if legend_loc is not None:
+                p.legend.location = legend_loc
 #        handle = None
 #        if show:
 #            handle = bp.show(p, notebook_handle=notebook_handle)
